# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import typing_extensions
from typing import Type, Iterable, cast

import httpx

from .sinks import (
    SinksResource,
    AsyncSinksResource,
    SinksResourceWithRawResponse,
    AsyncSinksResourceWithRawResponse,
    SinksResourceWithStreamingResponse,
    AsyncSinksResourceWithStreamingResponse,
)
from .streams import (
    StreamsResource,
    AsyncStreamsResource,
    StreamsResourceWithRawResponse,
    AsyncStreamsResourceWithRawResponse,
    StreamsResourceWithStreamingResponse,
    AsyncStreamsResourceWithStreamingResponse,
)
from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.pipelines import (
    pipeline_list_params,
    pipeline_create_params,
    pipeline_update_params,
    pipeline_list_v1_params,
    pipeline_create_v1_params,
    pipeline_validate_sql_params,
)
from ...types.pipelines.pipeline_get_response import PipelineGetResponse
from ...types.pipelines.pipeline_list_response import PipelineListResponse
from ...types.pipelines.pipeline_create_response import PipelineCreateResponse
from ...types.pipelines.pipeline_get_v1_response import PipelineGetV1Response
from ...types.pipelines.pipeline_update_response import PipelineUpdateResponse
from ...types.pipelines.pipeline_list_v1_response import PipelineListV1Response
from ...types.pipelines.pipeline_create_v1_response import PipelineCreateV1Response
from ...types.pipelines.pipeline_validate_sql_response import PipelineValidateSqlResponse

__all__ = ["PipelinesResource", "AsyncPipelinesResource"]


class PipelinesResource(SyncAPIResource):
    @cached_property
    def sinks(self) -> SinksResource:
        return SinksResource(self._client)

    @cached_property
    def streams(self) -> StreamsResource:
        return StreamsResource(self._client)

    @cached_property
    def with_raw_response(self) -> PipelinesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return PipelinesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> PipelinesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return PipelinesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("Use create_v1 instead. This endpoint will be removed in the future.")
    def create(
        self,
        *,
        account_id: str,
        destination: pipeline_create_params.Destination,
        name: str,
        source: Iterable[pipeline_create_params.Source],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineCreateResponse:
        """[DEPRECATED] Create a new pipeline.

        Use the new /pipelines/v1/pipelines endpoint
        instead.

        Args:
          account_id: Specifies the public ID of the account.

          name: Defines the name of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/pipelines", account_id=account_id),
            body=maybe_transform(
                {
                    "destination": destination,
                    "name": name,
                    "source": source,
                },
                pipeline_create_params.PipelineCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[PipelineCreateResponse], ResultWrapper[PipelineCreateResponse]),
        )

    @typing_extensions.deprecated("The v1 API does not support updates. This endpoint will be removed in the future.")
    def update(
        self,
        pipeline_name: str,
        *,
        account_id: str,
        destination: pipeline_update_params.Destination,
        name: str,
        source: Iterable[pipeline_update_params.Source],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineUpdateResponse:
        """[DEPRECATED] Update an existing pipeline.

        Use the new /pipelines/v1/pipelines
        endpoint instead.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_name: Defines the name of the pipeline.

          name: Defines the name of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_name:
            raise ValueError(f"Expected a non-empty value for `pipeline_name` but received {pipeline_name!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/pipelines/{pipeline_name}", account_id=account_id, pipeline_name=pipeline_name
            ),
            body=maybe_transform(
                {
                    "destination": destination,
                    "name": name,
                    "source": source,
                },
                pipeline_update_params.PipelineUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[PipelineUpdateResponse], ResultWrapper[PipelineUpdateResponse]),
        )

    @typing_extensions.deprecated("Use list_v1 instead. This endpoint will be removed in the future.")
    def list(
        self,
        *,
        account_id: str,
        page: str | Omit = omit,
        per_page: str | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineListResponse:
        """[DEPRECATED] List, filter, and paginate pipelines in an account.

        Use the new
        /pipelines/v1/pipelines endpoint instead.

        Args:
          account_id: Specifies the public ID of the account.

          page: Specifies which page to retrieve.

          per_page: Specifies the number of pipelines per page.

          search: Specifies the prefix of pipeline name to search.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/pipelines", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    pipeline_list_params.PipelineListParams,
                ),
            ),
            cast_to=PipelineListResponse,
        )

    @typing_extensions.deprecated("Use delete_v1 instead. This endpoint will be removed in the future.")
    def delete(
        self,
        pipeline_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """[DEPRECATED] Delete a pipeline.

        Use the new /pipelines/v1/pipelines endpoint
        instead.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_name: Defines the name of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_name:
            raise ValueError(f"Expected a non-empty value for `pipeline_name` but received {pipeline_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            path_template(
                "/accounts/{account_id}/pipelines/{pipeline_name}", account_id=account_id, pipeline_name=pipeline_name
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def create_v1(
        self,
        *,
        account_id: str,
        name: str,
        sql: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineCreateV1Response:
        """
        Create a new Pipeline.

        Args:
          account_id: Specifies the public ID of the account.

          name: Specifies the name of the Pipeline.

          sql: Specifies SQL for the Pipeline processing flow.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/pipelines/v1/pipelines", account_id=account_id),
            body=maybe_transform(
                {
                    "name": name,
                    "sql": sql,
                },
                pipeline_create_v1_params.PipelineCreateV1Params,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineCreateV1Response]._unwrapper,
            ),
            cast_to=cast(Type[PipelineCreateV1Response], ResultWrapper[PipelineCreateV1Response]),
        )

    def delete_v1(
        self,
        pipeline_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Pipeline in Account.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_id: Specifies the public ID of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/pipelines/v1/pipelines/{pipeline_id}",
                account_id=account_id,
                pipeline_id=pipeline_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    @typing_extensions.deprecated("Use get_v1 instead. This endpoint will be removed in the future.")
    def get(
        self,
        pipeline_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineGetResponse:
        """[DEPRECATED] Get configuration of a pipeline.

        Use the new
        /pipelines/v1/pipelines endpoint instead.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_name: Defines the name of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_name:
            raise ValueError(f"Expected a non-empty value for `pipeline_name` but received {pipeline_name!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/pipelines/{pipeline_name}", account_id=account_id, pipeline_name=pipeline_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[PipelineGetResponse], ResultWrapper[PipelineGetResponse]),
        )

    def get_v1(
        self,
        pipeline_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineGetV1Response:
        """
        Get Pipelines Details.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_id: Specifies the public ID of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/pipelines/v1/pipelines/{pipeline_id}",
                account_id=account_id,
                pipeline_id=pipeline_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineGetV1Response]._unwrapper,
            ),
            cast_to=cast(Type[PipelineGetV1Response], ResultWrapper[PipelineGetV1Response]),
        )

    def list_v1(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[PipelineListV1Response]:
        """
        List/Filter Pipelines in Account.

        Args:
          account_id: Specifies the public ID of the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/pipelines/v1/pipelines", account_id=account_id),
            page=SyncV4PagePaginationArray[PipelineListV1Response],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    pipeline_list_v1_params.PipelineListV1Params,
                ),
            ),
            model=PipelineListV1Response,
        )

    def validate_sql(
        self,
        *,
        account_id: str,
        sql: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineValidateSqlResponse:
        """
        Validate Arroyo SQL.

        Args:
          account_id: Specifies the public ID of the account.

          sql: Specifies SQL to validate.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/pipelines/v1/validate_sql", account_id=account_id),
            body=maybe_transform({"sql": sql}, pipeline_validate_sql_params.PipelineValidateSqlParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineValidateSqlResponse]._unwrapper,
            ),
            cast_to=cast(Type[PipelineValidateSqlResponse], ResultWrapper[PipelineValidateSqlResponse]),
        )


class AsyncPipelinesResource(AsyncAPIResource):
    @cached_property
    def sinks(self) -> AsyncSinksResource:
        return AsyncSinksResource(self._client)

    @cached_property
    def streams(self) -> AsyncStreamsResource:
        return AsyncStreamsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncPipelinesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncPipelinesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncPipelinesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncPipelinesResourceWithStreamingResponse(self)

    @typing_extensions.deprecated("Use create_v1 instead. This endpoint will be removed in the future.")
    async def create(
        self,
        *,
        account_id: str,
        destination: pipeline_create_params.Destination,
        name: str,
        source: Iterable[pipeline_create_params.Source],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineCreateResponse:
        """[DEPRECATED] Create a new pipeline.

        Use the new /pipelines/v1/pipelines endpoint
        instead.

        Args:
          account_id: Specifies the public ID of the account.

          name: Defines the name of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/pipelines", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "destination": destination,
                    "name": name,
                    "source": source,
                },
                pipeline_create_params.PipelineCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[PipelineCreateResponse], ResultWrapper[PipelineCreateResponse]),
        )

    @typing_extensions.deprecated("The v1 API does not support updates. This endpoint will be removed in the future.")
    async def update(
        self,
        pipeline_name: str,
        *,
        account_id: str,
        destination: pipeline_update_params.Destination,
        name: str,
        source: Iterable[pipeline_update_params.Source],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineUpdateResponse:
        """[DEPRECATED] Update an existing pipeline.

        Use the new /pipelines/v1/pipelines
        endpoint instead.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_name: Defines the name of the pipeline.

          name: Defines the name of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_name:
            raise ValueError(f"Expected a non-empty value for `pipeline_name` but received {pipeline_name!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/pipelines/{pipeline_name}", account_id=account_id, pipeline_name=pipeline_name
            ),
            body=await async_maybe_transform(
                {
                    "destination": destination,
                    "name": name,
                    "source": source,
                },
                pipeline_update_params.PipelineUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[PipelineUpdateResponse], ResultWrapper[PipelineUpdateResponse]),
        )

    @typing_extensions.deprecated("Use list_v1 instead. This endpoint will be removed in the future.")
    async def list(
        self,
        *,
        account_id: str,
        page: str | Omit = omit,
        per_page: str | Omit = omit,
        search: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineListResponse:
        """[DEPRECATED] List, filter, and paginate pipelines in an account.

        Use the new
        /pipelines/v1/pipelines endpoint instead.

        Args:
          account_id: Specifies the public ID of the account.

          page: Specifies which page to retrieve.

          per_page: Specifies the number of pipelines per page.

          search: Specifies the prefix of pipeline name to search.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/pipelines", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                        "search": search,
                    },
                    pipeline_list_params.PipelineListParams,
                ),
            ),
            cast_to=PipelineListResponse,
        )

    @typing_extensions.deprecated("Use delete_v1 instead. This endpoint will be removed in the future.")
    async def delete(
        self,
        pipeline_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """[DEPRECATED] Delete a pipeline.

        Use the new /pipelines/v1/pipelines endpoint
        instead.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_name: Defines the name of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_name:
            raise ValueError(f"Expected a non-empty value for `pipeline_name` but received {pipeline_name!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            path_template(
                "/accounts/{account_id}/pipelines/{pipeline_name}", account_id=account_id, pipeline_name=pipeline_name
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def create_v1(
        self,
        *,
        account_id: str,
        name: str,
        sql: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineCreateV1Response:
        """
        Create a new Pipeline.

        Args:
          account_id: Specifies the public ID of the account.

          name: Specifies the name of the Pipeline.

          sql: Specifies SQL for the Pipeline processing flow.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/pipelines/v1/pipelines", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "name": name,
                    "sql": sql,
                },
                pipeline_create_v1_params.PipelineCreateV1Params,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineCreateV1Response]._unwrapper,
            ),
            cast_to=cast(Type[PipelineCreateV1Response], ResultWrapper[PipelineCreateV1Response]),
        )

    async def delete_v1(
        self,
        pipeline_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete Pipeline in Account.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_id: Specifies the public ID of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/pipelines/v1/pipelines/{pipeline_id}",
                account_id=account_id,
                pipeline_id=pipeline_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )

    @typing_extensions.deprecated("Use get_v1 instead. This endpoint will be removed in the future.")
    async def get(
        self,
        pipeline_name: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineGetResponse:
        """[DEPRECATED] Get configuration of a pipeline.

        Use the new
        /pipelines/v1/pipelines endpoint instead.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_name: Defines the name of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_name:
            raise ValueError(f"Expected a non-empty value for `pipeline_name` but received {pipeline_name!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/pipelines/{pipeline_name}", account_id=account_id, pipeline_name=pipeline_name
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[PipelineGetResponse], ResultWrapper[PipelineGetResponse]),
        )

    async def get_v1(
        self,
        pipeline_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineGetV1Response:
        """
        Get Pipelines Details.

        Args:
          account_id: Specifies the public ID of the account.

          pipeline_id: Specifies the public ID of the pipeline.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not pipeline_id:
            raise ValueError(f"Expected a non-empty value for `pipeline_id` but received {pipeline_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/pipelines/v1/pipelines/{pipeline_id}",
                account_id=account_id,
                pipeline_id=pipeline_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineGetV1Response]._unwrapper,
            ),
            cast_to=cast(Type[PipelineGetV1Response], ResultWrapper[PipelineGetV1Response]),
        )

    def list_v1(
        self,
        *,
        account_id: str,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[PipelineListV1Response, AsyncV4PagePaginationArray[PipelineListV1Response]]:
        """
        List/Filter Pipelines in Account.

        Args:
          account_id: Specifies the public ID of the account.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/pipelines/v1/pipelines", account_id=account_id),
            page=AsyncV4PagePaginationArray[PipelineListV1Response],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "per_page": per_page,
                    },
                    pipeline_list_v1_params.PipelineListV1Params,
                ),
            ),
            model=PipelineListV1Response,
        )

    async def validate_sql(
        self,
        *,
        account_id: str,
        sql: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> PipelineValidateSqlResponse:
        """
        Validate Arroyo SQL.

        Args:
          account_id: Specifies the public ID of the account.

          sql: Specifies SQL to validate.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/pipelines/v1/validate_sql", account_id=account_id),
            body=await async_maybe_transform({"sql": sql}, pipeline_validate_sql_params.PipelineValidateSqlParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[PipelineValidateSqlResponse]._unwrapper,
            ),
            cast_to=cast(Type[PipelineValidateSqlResponse], ResultWrapper[PipelineValidateSqlResponse]),
        )


class PipelinesResourceWithRawResponse:
    def __init__(self, pipelines: PipelinesResource) -> None:
        self._pipelines = pipelines

        self.create = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                pipelines.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.update = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                pipelines.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                pipelines.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                pipelines.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.create_v1 = to_raw_response_wrapper(
            pipelines.create_v1,
        )
        self.delete_v1 = to_raw_response_wrapper(
            pipelines.delete_v1,
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_raw_response_wrapper(
                pipelines.get,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get_v1 = to_raw_response_wrapper(
            pipelines.get_v1,
        )
        self.list_v1 = to_raw_response_wrapper(
            pipelines.list_v1,
        )
        self.validate_sql = to_raw_response_wrapper(
            pipelines.validate_sql,
        )

    @cached_property
    def sinks(self) -> SinksResourceWithRawResponse:
        return SinksResourceWithRawResponse(self._pipelines.sinks)

    @cached_property
    def streams(self) -> StreamsResourceWithRawResponse:
        return StreamsResourceWithRawResponse(self._pipelines.streams)


class AsyncPipelinesResourceWithRawResponse:
    def __init__(self, pipelines: AsyncPipelinesResource) -> None:
        self._pipelines = pipelines

        self.create = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                pipelines.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.update = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                pipelines.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                pipelines.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                pipelines.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.create_v1 = async_to_raw_response_wrapper(
            pipelines.create_v1,
        )
        self.delete_v1 = async_to_raw_response_wrapper(
            pipelines.delete_v1,
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_raw_response_wrapper(
                pipelines.get,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get_v1 = async_to_raw_response_wrapper(
            pipelines.get_v1,
        )
        self.list_v1 = async_to_raw_response_wrapper(
            pipelines.list_v1,
        )
        self.validate_sql = async_to_raw_response_wrapper(
            pipelines.validate_sql,
        )

    @cached_property
    def sinks(self) -> AsyncSinksResourceWithRawResponse:
        return AsyncSinksResourceWithRawResponse(self._pipelines.sinks)

    @cached_property
    def streams(self) -> AsyncStreamsResourceWithRawResponse:
        return AsyncStreamsResourceWithRawResponse(self._pipelines.streams)


class PipelinesResourceWithStreamingResponse:
    def __init__(self, pipelines: PipelinesResource) -> None:
        self._pipelines = pipelines

        self.create = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                pipelines.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.update = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                pipelines.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                pipelines.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                pipelines.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.create_v1 = to_streamed_response_wrapper(
            pipelines.create_v1,
        )
        self.delete_v1 = to_streamed_response_wrapper(
            pipelines.delete_v1,
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            to_streamed_response_wrapper(
                pipelines.get,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get_v1 = to_streamed_response_wrapper(
            pipelines.get_v1,
        )
        self.list_v1 = to_streamed_response_wrapper(
            pipelines.list_v1,
        )
        self.validate_sql = to_streamed_response_wrapper(
            pipelines.validate_sql,
        )

    @cached_property
    def sinks(self) -> SinksResourceWithStreamingResponse:
        return SinksResourceWithStreamingResponse(self._pipelines.sinks)

    @cached_property
    def streams(self) -> StreamsResourceWithStreamingResponse:
        return StreamsResourceWithStreamingResponse(self._pipelines.streams)


class AsyncPipelinesResourceWithStreamingResponse:
    def __init__(self, pipelines: AsyncPipelinesResource) -> None:
        self._pipelines = pipelines

        self.create = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                pipelines.create,  # pyright: ignore[reportDeprecated],
            )
        )
        self.update = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                pipelines.update,  # pyright: ignore[reportDeprecated],
            )
        )
        self.list = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                pipelines.list,  # pyright: ignore[reportDeprecated],
            )
        )
        self.delete = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                pipelines.delete,  # pyright: ignore[reportDeprecated],
            )
        )
        self.create_v1 = async_to_streamed_response_wrapper(
            pipelines.create_v1,
        )
        self.delete_v1 = async_to_streamed_response_wrapper(
            pipelines.delete_v1,
        )
        self.get = (  # pyright: ignore[reportDeprecated]
            async_to_streamed_response_wrapper(
                pipelines.get,  # pyright: ignore[reportDeprecated],
            )
        )
        self.get_v1 = async_to_streamed_response_wrapper(
            pipelines.get_v1,
        )
        self.list_v1 = async_to_streamed_response_wrapper(
            pipelines.list_v1,
        )
        self.validate_sql = async_to_streamed_response_wrapper(
            pipelines.validate_sql,
        )

    @cached_property
    def sinks(self) -> AsyncSinksResourceWithStreamingResponse:
        return AsyncSinksResourceWithStreamingResponse(self._pipelines.sinks)

    @cached_property
    def streams(self) -> AsyncStreamsResourceWithStreamingResponse:
        return AsyncStreamsResourceWithStreamingResponse(self._pipelines.streams)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["RecipientCreateParams"]


class RecipientCreateParams(TypedDict, total=False):
    path_account_id: Required[Annotated[str, PropertyInfo(alias="account_id")]]
    """Account identifier."""

    body_account_id: Annotated[str, PropertyInfo(alias="account_id")]
    """Account identifier."""

    organization_id: str
    """Organization identifier."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["SSLParam"]


class SSLParam(TypedDict, total=False):
    id: Literal["ssl"]
    """
    Control options for the SSL feature of the Edge Certificates tab in the
    Cloudflare SSL/TLS app.
    """

    value: Literal["off", "flexible", "full", "strict", "origin_pull"]
    """The encryption mode that Cloudflare uses to connect to your origin server."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .events import (
    EventsResource,
    AsyncEventsResource,
    EventsResourceWithRawResponse,
    AsyncEventsResourceWithRawResponse,
    EventsResourceWithStreamingResponse,
    AsyncEventsResourceWithStreamingResponse,
)
from .snapshots import (
    SnapshotsResource,
    AsyncSnapshotsResource,
    SnapshotsResourceWithRawResponse,
    AsyncSnapshotsResourceWithRawResponse,
    SnapshotsResourceWithStreamingResponse,
    AsyncSnapshotsResourceWithStreamingResponse,
)
from .connectors import (
    ConnectorsResource,
    AsyncConnectorsResource,
    ConnectorsResourceWithRawResponse,
    AsyncConnectorsResourceWithRawResponse,
    ConnectorsResourceWithStreamingResponse,
    AsyncConnectorsResourceWithStreamingResponse,
)

__all__ = [
    "EventsResource",
    "AsyncEventsResource",
    "EventsResourceWithRawResponse",
    "AsyncEventsResourceWithRawResponse",
    "EventsResourceWithStreamingResponse",
    "AsyncEventsResourceWithStreamingResponse",
    "SnapshotsResource",
    "AsyncSnapshotsResource",
    "SnapshotsResourceWithRawResponse",
    "AsyncSnapshotsResourceWithRawResponse",
    "SnapshotsResourceWithStreamingResponse",
    "AsyncSnapshotsResourceWithStreamingResponse",
    "ConnectorsResource",
    "AsyncConnectorsResource",
    "ConnectorsResourceWithRawResponse",
    "AsyncConnectorsResourceWithRawResponse",
    "ConnectorsResourceWithStreamingResponse",
    "AsyncConnectorsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["OpportunisticEncryption"]


class OpportunisticEncryption(BaseModel):
    id: Optional[Literal["opportunistic_encryption"]] = None
    """
    Opportunistic Encryption allows browsers to access HTTP URIs over an encrypted
    TLS channel. It's not a substitute for HTTPS, but provides additional security
    for otherwise vulnerable requests.
    """

    value: Optional[Literal["on", "off"]] = None
    """The status of Opportunistic Encryption."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["Pagerduty"]


class Pagerduty(BaseModel):
    id: Optional[str] = None
    """UUID"""

    name: Optional[str] = None
    """The name of the pagerduty service."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .direct_upload_create_params import DirectUploadCreateParams as DirectUploadCreateParams
from .direct_upload_create_response import DirectUploadCreateResponse as DirectUploadCreateResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["OrganizationProfileUpdateParams"]


class OrganizationProfileUpdateParams(TypedDict, total=False):
    business_address: Required[str]

    business_email: Required[str]

    business_name: Required[str]

    business_phone: Required[str]

    external_metadata: Required[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .catalog_syncs import (
    CatalogSyncsResource,
    AsyncCatalogSyncsResource,
    CatalogSyncsResourceWithRawResponse,
    AsyncCatalogSyncsResourceWithRawResponse,
    CatalogSyncsResourceWithStreamingResponse,
    AsyncCatalogSyncsResourceWithStreamingResponse,
)
from .prebuilt_policies import (
    PrebuiltPoliciesResource,
    AsyncPrebuiltPoliciesResource,
    PrebuiltPoliciesResourceWithRawResponse,
    AsyncPrebuiltPoliciesResourceWithRawResponse,
    PrebuiltPoliciesResourceWithStreamingResponse,
    AsyncPrebuiltPoliciesResourceWithStreamingResponse,
)

__all__ = [
    "PrebuiltPoliciesResource",
    "AsyncPrebuiltPoliciesResource",
    "PrebuiltPoliciesResourceWithRawResponse",
    "AsyncPrebuiltPoliciesResourceWithRawResponse",
    "PrebuiltPoliciesResourceWithStreamingResponse",
    "AsyncPrebuiltPoliciesResourceWithStreamingResponse",
    "CatalogSyncsResource",
    "AsyncCatalogSyncsResource",
    "CatalogSyncsResourceWithRawResponse",
    "AsyncCatalogSyncsResourceWithRawResponse",
    "CatalogSyncsResourceWithStreamingResponse",
    "AsyncCatalogSyncsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ...._compat import (  # type: ignore[attr-defined]
    PYDANTIC_V2,  # pyright: ignore[reportAttributeAccessIssue, reportUnknownVariableType]
)
from ...._models import BaseModel

__all__ = [
    "Profile",
    "CustomProfile",
    "CustomProfileEntry",
    "CustomProfileEntryCustomEntry",
    "CustomProfileEntryCustomPromptTopicEntry",
    "CustomProfileEntryPredefinedEntry",
    "CustomProfileEntryPredefinedEntryConfidence",
    "CustomProfileEntryPredefinedEntryVariant",
    "CustomProfileEntryPredefinedEntryVariantUnionMember0",
    "CustomProfileEntryPredefinedEntryVariantUnionMember1",
    "CustomProfileEntryIntegrationEntry",
    "CustomProfileEntryExactDataEntry",
    "CustomProfileEntryDocumentFingerprintEntry",
    "CustomProfileEntryWordListEntry",
    "CustomProfileSensitivityLevel",
    "CustomProfileSharedEntry",
    "CustomProfileSharedEntryCustomEntry",
    "CustomProfileSharedEntryCustomPromptTopicEntry",
    "CustomProfileSharedEntryPredefinedEntry",
    "CustomProfileSharedEntryPredefinedEntryConfidence",
    "CustomProfileSharedEntryPredefinedEntryVariant",
    "CustomProfileSharedEntryPredefinedEntryVariantUnionMember0",
    "CustomProfileSharedEntryPredefinedEntryVariantUnionMember1",
    "CustomProfileSharedEntryIntegrationEntry",
    "CustomProfileSharedEntryExactDataEntry",
    "CustomProfileSharedEntryDocumentFingerprintEntry",
    "CustomProfileSharedEntryWordListEntry",
    "PredefinedProfile",
    "PredefinedProfileEntry",
    "PredefinedProfileEntryCustomEntry",
    "PredefinedProfileEntryCustomPromptTopicEntry",
    "PredefinedProfileEntryPredefinedEntry",
    "PredefinedProfileEntryPredefinedEntryConfidence",
    "PredefinedProfileEntryPredefinedEntryVariant",
    "PredefinedProfileEntryPredefinedEntryVariantUnionMember0",
    "PredefinedProfileEntryPredefinedEntryVariantUnionMember1",
    "PredefinedProfileEntryIntegrationEntry",
    "PredefinedProfileEntryExactDataEntry",
    "PredefinedProfileEntryDocumentFingerprintEntry",
    "PredefinedProfileEntryWordListEntry",
    "IntegrationProfile",
    "IntegrationProfileEntry",
    "IntegrationProfileEntryCustomEntry",
    "IntegrationProfileEntryCustomPromptTopicEntry",
    "IntegrationProfileEntryPredefinedEntry",
    "IntegrationProfileEntryPredefinedEntryConfidence",
    "IntegrationProfileEntryPredefinedEntryVariant",
    "IntegrationProfileEntryPredefinedEntryVariantUnionMember0",
    "IntegrationProfileEntryPredefinedEntryVariantUnionMember1",
    "IntegrationProfileEntryIntegrationEntry",
    "IntegrationProfileEntryExactDataEntry",
    "IntegrationProfileEntryDocumentFingerprintEntry",
    "IntegrationProfileEntryWordListEntry",
    "IntegrationProfileSharedEntry",
    "IntegrationProfileSharedEntryCustomEntry",
    "IntegrationProfileSharedEntryCustomPromptTopicEntry",
    "IntegrationProfileSharedEntryPredefinedEntry",
    "IntegrationProfileSharedEntryPredefinedEntryConfidence",
    "IntegrationProfileSharedEntryPredefinedEntryVariant",
    "IntegrationProfileSharedEntryPredefinedEntryVariantUnionMember0",
    "IntegrationProfileSharedEntryPredefinedEntryVariantUnionMember1",
    "IntegrationProfileSharedEntryIntegrationEntry",
    "IntegrationProfileSharedEntryExactDataEntry",
    "IntegrationProfileSharedEntryDocumentFingerprintEntry",
    "IntegrationProfileSharedEntryWordListEntry",
]


class CustomProfileEntryCustomEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    pattern: Pattern

    type: Literal["custom"]

    updated_at: datetime

    description: Optional[str] = None

    profile_id: Optional[str] = None


class CustomProfileEntryCustomPromptTopicEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["custom_prompt_topic"]

    updated_at: datetime


class CustomProfileEntryPredefinedEntryConfidence(BaseModel):
    ai_context_available: bool
    """Indicates whether this entry has AI remote service validation."""

    available: bool
    """
    Indicates whether this entry has any form of validation that is not an AI remote
    service.
    """


class CustomProfileEntryPredefinedEntryVariantUnionMember0(BaseModel):
    """A Predefined AI prompt classification topic entry."""

    topic_type: Literal["Intent", "Content"]

    type: Literal["PromptTopic"]

    description: Optional[str] = None
    """
    A customer-facing explanation of what this predefined AI prompt topic
    represents.
    """


class CustomProfileEntryPredefinedEntryVariantUnionMember1(BaseModel):
    """A general predefined entry."""

    type: Literal["General"]

    description: Optional[str] = None
    """A customer-facing explanation of what this predefined entry represents."""


CustomProfileEntryPredefinedEntryVariant: TypeAlias = Union[
    CustomProfileEntryPredefinedEntryVariantUnionMember0, CustomProfileEntryPredefinedEntryVariantUnionMember1
]


class CustomProfileEntryPredefinedEntry(BaseModel):
    id: str

    confidence: CustomProfileEntryPredefinedEntryConfidence

    enabled: bool

    name: str

    type: Literal["predefined"]

    profile_id: Optional[str] = None

    variant: Optional[CustomProfileEntryPredefinedEntryVariant] = None
    """A Predefined AI prompt classification topic entry."""


class CustomProfileEntryIntegrationEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["integration"]

    updated_at: datetime

    profile_id: Optional[str] = None


class CustomProfileEntryExactDataEntry(BaseModel):
    id: str

    case_sensitive: bool
    """
    Only applies to custom word lists. Determines if the words should be matched in
    a case-sensitive manner Cannot be set to false if secret is true
    """

    created_at: datetime

    enabled: bool

    name: str

    secret: bool

    type: Literal["exact_data"]

    updated_at: datetime


class CustomProfileEntryDocumentFingerprintEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["document_fingerprint"]

    updated_at: datetime


class CustomProfileEntryWordListEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["word_list"]

    updated_at: datetime

    word_list: object

    profile_id: Optional[str] = None


CustomProfileEntry: TypeAlias = Union[
    CustomProfileEntryCustomEntry,
    CustomProfileEntryCustomPromptTopicEntry,
    CustomProfileEntryPredefinedEntry,
    CustomProfileEntryIntegrationEntry,
    CustomProfileEntryExactDataEntry,
    CustomProfileEntryDocumentFingerprintEntry,
    CustomProfileEntryWordListEntry,
]


class CustomProfileSensitivityLevel(BaseModel):
    """
    A reference pairing a sensitivity group with a specific level within that group.
    """

    group_id: str

    level_id: str


class CustomProfileSharedEntryCustomEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    pattern: Pattern

    type: Literal["custom"]

    updated_at: datetime

    description: Optional[str] = None

    profile_id: Optional[str] = None


class CustomProfileSharedEntryCustomPromptTopicEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["custom_prompt_topic"]

    updated_at: datetime


class CustomProfileSharedEntryPredefinedEntryConfidence(BaseModel):
    ai_context_available: bool
    """Indicates whether this entry has AI remote service validation."""

    available: bool
    """
    Indicates whether this entry has any form of validation that is not an AI remote
    service.
    """


class CustomProfileSharedEntryPredefinedEntryVariantUnionMember0(BaseModel):
    """A Predefined AI prompt classification topic entry."""

    topic_type: Literal["Intent", "Content"]

    type: Literal["PromptTopic"]

    description: Optional[str] = None
    """
    A customer-facing explanation of what this predefined AI prompt topic
    represents.
    """


class CustomProfileSharedEntryPredefinedEntryVariantUnionMember1(BaseModel):
    """A general predefined entry."""

    type: Literal["General"]

    description: Optional[str] = None
    """A customer-facing explanation of what this predefined entry represents."""


CustomProfileSharedEntryPredefinedEntryVariant: TypeAlias = Union[
    CustomProfileSharedEntryPredefinedEntryVariantUnionMember0,
    CustomProfileSharedEntryPredefinedEntryVariantUnionMember1,
]


class CustomProfileSharedEntryPredefinedEntry(BaseModel):
    id: str

    confidence: CustomProfileSharedEntryPredefinedEntryConfidence

    enabled: bool

    name: str

    type: Literal["predefined"]

    profile_id: Optional[str] = None

    variant: Optional[CustomProfileSharedEntryPredefinedEntryVariant] = None
    """A Predefined AI prompt classification topic entry."""


class CustomProfileSharedEntryIntegrationEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["integration"]

    updated_at: datetime

    profile_id: Optional[str] = None


class CustomProfileSharedEntryExactDataEntry(BaseModel):
    id: str

    case_sensitive: bool
    """
    Only applies to custom word lists. Determines if the words should be matched in
    a case-sensitive manner Cannot be set to false if secret is true
    """

    created_at: datetime

    enabled: bool

    name: str

    secret: bool

    type: Literal["exact_data"]

    updated_at: datetime


class CustomProfileSharedEntryDocumentFingerprintEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["document_fingerprint"]

    updated_at: datetime


class CustomProfileSharedEntryWordListEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["word_list"]

    updated_at: datetime

    word_list: object

    profile_id: Optional[str] = None


CustomProfileSharedEntry: TypeAlias = Union[
    CustomProfileSharedEntryCustomEntry,
    CustomProfileSharedEntryCustomPromptTopicEntry,
    CustomProfileSharedEntryPredefinedEntry,
    CustomProfileSharedEntryIntegrationEntry,
    CustomProfileSharedEntryExactDataEntry,
    CustomProfileSharedEntryDocumentFingerprintEntry,
    CustomProfileSharedEntryWordListEntry,
]


class CustomProfile(BaseModel):
    id: str
    """The id of the profile (uuid)."""

    allowed_match_count: int
    """Related DLP policies will trigger when the match count exceeds the number set."""

    created_at: datetime
    """When the profile was created."""

    name: str
    """The name of the profile."""

    ocr_enabled: bool

    type: Literal["custom"]

    updated_at: datetime
    """When the profile was lasted updated."""

    ai_context_enabled: Optional[bool] = None

    confidence_threshold: Optional[Literal["low", "medium", "high", "very_high"]] = None

    context_awareness: Optional[ContextAwareness] = None
    """
    Scan the context of predefined entries to only return matches surrounded by
    keywords.
    """

    data_classes: Optional[List[str]] = None
    """Data classes associated with this profile."""

    data_tags: Optional[List[str]] = None
    """Data tags associated with this profile."""

    description: Optional[str] = None
    """The description of the profile."""

    entries: Optional[List[CustomProfileEntry]] = None

    sensitivity_levels: Optional[List[CustomProfileSensitivityLevel]] = None
    """Sensitivity levels associated with this profile."""

    shared_entries: Optional[List[CustomProfileSharedEntry]] = None


class PredefinedProfileEntryCustomEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    pattern: Pattern

    type: Literal["custom"]

    updated_at: datetime

    description: Optional[str] = None

    profile_id: Optional[str] = None


class PredefinedProfileEntryCustomPromptTopicEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["custom_prompt_topic"]

    updated_at: datetime


class PredefinedProfileEntryPredefinedEntryConfidence(BaseModel):
    ai_context_available: bool
    """Indicates whether this entry has AI remote service validation."""

    available: bool
    """
    Indicates whether this entry has any form of validation that is not an AI remote
    service.
    """


class PredefinedProfileEntryPredefinedEntryVariantUnionMember0(BaseModel):
    """A Predefined AI prompt classification topic entry."""

    topic_type: Literal["Intent", "Content"]

    type: Literal["PromptTopic"]

    description: Optional[str] = None
    """
    A customer-facing explanation of what this predefined AI prompt topic
    represents.
    """


class PredefinedProfileEntryPredefinedEntryVariantUnionMember1(BaseModel):
    """A general predefined entry."""

    type: Literal["General"]

    description: Optional[str] = None
    """A customer-facing explanation of what this predefined entry represents."""


PredefinedProfileEntryPredefinedEntryVariant: TypeAlias = Union[
    PredefinedProfileEntryPredefinedEntryVariantUnionMember0, PredefinedProfileEntryPredefinedEntryVariantUnionMember1
]


class PredefinedProfileEntryPredefinedEntry(BaseModel):
    id: str

    confidence: PredefinedProfileEntryPredefinedEntryConfidence

    enabled: bool

    name: str

    type: Literal["predefined"]

    profile_id: Optional[str] = None

    variant: Optional[PredefinedProfileEntryPredefinedEntryVariant] = None
    """A Predefined AI prompt classification topic entry."""


class PredefinedProfileEntryIntegrationEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["integration"]

    updated_at: datetime

    profile_id: Optional[str] = None


class PredefinedProfileEntryExactDataEntry(BaseModel):
    id: str

    case_sensitive: bool
    """
    Only applies to custom word lists. Determines if the words should be matched in
    a case-sensitive manner Cannot be set to false if secret is true
    """

    created_at: datetime

    enabled: bool

    name: str

    secret: bool

    type: Literal["exact_data"]

    updated_at: datetime


class PredefinedProfileEntryDocumentFingerprintEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["document_fingerprint"]

    updated_at: datetime


class PredefinedProfileEntryWordListEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["word_list"]

    updated_at: datetime

    word_list: object

    profile_id: Optional[str] = None


PredefinedProfileEntry: TypeAlias = Union[
    PredefinedProfileEntryCustomEntry,
    PredefinedProfileEntryCustomPromptTopicEntry,
    PredefinedProfileEntryPredefinedEntry,
    PredefinedProfileEntryIntegrationEntry,
    PredefinedProfileEntryExactDataEntry,
    PredefinedProfileEntryDocumentFingerprintEntry,
    PredefinedProfileEntryWordListEntry,
]


class PredefinedProfile(BaseModel):
    id: str
    """The id of the predefined profile (uuid)."""

    allowed_match_count: int

    entries: List[PredefinedProfileEntry]

    name: str
    """The name of the predefined profile."""

    type: Literal["predefined"]

    ai_context_enabled: Optional[bool] = None

    confidence_threshold: Optional[Literal["low", "medium", "high", "very_high"]] = None

    context_awareness: Optional[ContextAwareness] = None
    """
    Scan the context of predefined entries to only return matches surrounded by
    keywords.
    """

    ocr_enabled: Optional[bool] = None

    open_access: Optional[bool] = None
    """Whether this profile can be accessed by anyone."""


class IntegrationProfileEntryCustomEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    pattern: Pattern

    type: Literal["custom"]

    updated_at: datetime

    description: Optional[str] = None

    profile_id: Optional[str] = None


class IntegrationProfileEntryCustomPromptTopicEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["custom_prompt_topic"]

    updated_at: datetime


class IntegrationProfileEntryPredefinedEntryConfidence(BaseModel):
    ai_context_available: bool
    """Indicates whether this entry has AI remote service validation."""

    available: bool
    """
    Indicates whether this entry has any form of validation that is not an AI remote
    service.
    """


class IntegrationProfileEntryPredefinedEntryVariantUnionMember0(BaseModel):
    """A Predefined AI prompt classification topic entry."""

    topic_type: Literal["Intent", "Content"]

    type: Literal["PromptTopic"]

    description: Optional[str] = None
    """
    A customer-facing explanation of what this predefined AI prompt topic
    represents.
    """


class IntegrationProfileEntryPredefinedEntryVariantUnionMember1(BaseModel):
    """A general predefined entry."""

    type: Literal["General"]

    description: Optional[str] = None
    """A customer-facing explanation of what this predefined entry represents."""


IntegrationProfileEntryPredefinedEntryVariant: TypeAlias = Union[
    IntegrationProfileEntryPredefinedEntryVariantUnionMember0, IntegrationProfileEntryPredefinedEntryVariantUnionMember1
]


class IntegrationProfileEntryPredefinedEntry(BaseModel):
    id: str

    confidence: IntegrationProfileEntryPredefinedEntryConfidence

    enabled: bool

    name: str

    type: Literal["predefined"]

    profile_id: Optional[str] = None

    variant: Optional[IntegrationProfileEntryPredefinedEntryVariant] = None
    """A Predefined AI prompt classification topic entry."""


class IntegrationProfileEntryIntegrationEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["integration"]

    updated_at: datetime

    profile_id: Optional[str] = None


class IntegrationProfileEntryExactDataEntry(BaseModel):
    id: str

    case_sensitive: bool
    """
    Only applies to custom word lists. Determines if the words should be matched in
    a case-sensitive manner Cannot be set to false if secret is true
    """

    created_at: datetime

    enabled: bool

    name: str

    secret: bool

    type: Literal["exact_data"]

    updated_at: datetime


class IntegrationProfileEntryDocumentFingerprintEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["document_fingerprint"]

    updated_at: datetime


class IntegrationProfileEntryWordListEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["word_list"]

    updated_at: datetime

    word_list: object

    profile_id: Optional[str] = None


IntegrationProfileEntry: TypeAlias = Union[
    IntegrationProfileEntryCustomEntry,
    IntegrationProfileEntryCustomPromptTopicEntry,
    IntegrationProfileEntryPredefinedEntry,
    IntegrationProfileEntryIntegrationEntry,
    IntegrationProfileEntryExactDataEntry,
    IntegrationProfileEntryDocumentFingerprintEntry,
    IntegrationProfileEntryWordListEntry,
]


class IntegrationProfileSharedEntryCustomEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    pattern: Pattern

    type: Literal["custom"]

    updated_at: datetime

    description: Optional[str] = None

    profile_id: Optional[str] = None


class IntegrationProfileSharedEntryCustomPromptTopicEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["custom_prompt_topic"]

    updated_at: datetime


class IntegrationProfileSharedEntryPredefinedEntryConfidence(BaseModel):
    ai_context_available: bool
    """Indicates whether this entry has AI remote service validation."""

    available: bool
    """
    Indicates whether this entry has any form of validation that is not an AI remote
    service.
    """


class IntegrationProfileSharedEntryPredefinedEntryVariantUnionMember0(BaseModel):
    """A Predefined AI prompt classification topic entry."""

    topic_type: Literal["Intent", "Content"]

    type: Literal["PromptTopic"]

    description: Optional[str] = None
    """
    A customer-facing explanation of what this predefined AI prompt topic
    represents.
    """


class IntegrationProfileSharedEntryPredefinedEntryVariantUnionMember1(BaseModel):
    """A general predefined entry."""

    type: Literal["General"]

    description: Optional[str] = None
    """A customer-facing explanation of what this predefined entry represents."""


IntegrationProfileSharedEntryPredefinedEntryVariant: TypeAlias = Union[
    IntegrationProfileSharedEntryPredefinedEntryVariantUnionMember0,
    IntegrationProfileSharedEntryPredefinedEntryVariantUnionMember1,
]


class IntegrationProfileSharedEntryPredefinedEntry(BaseModel):
    id: str

    confidence: IntegrationProfileSharedEntryPredefinedEntryConfidence

    enabled: bool

    name: str

    type: Literal["predefined"]

    profile_id: Optional[str] = None

    variant: Optional[IntegrationProfileSharedEntryPredefinedEntryVariant] = None
    """A Predefined AI prompt classification topic entry."""


class IntegrationProfileSharedEntryIntegrationEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["integration"]

    updated_at: datetime

    profile_id: Optional[str] = None


class IntegrationProfileSharedEntryExactDataEntry(BaseModel):
    id: str

    case_sensitive: bool
    """
    Only applies to custom word lists. Determines if the words should be matched in
    a case-sensitive manner Cannot be set to false if secret is true
    """

    created_at: datetime

    enabled: bool

    name: str

    secret: bool

    type: Literal["exact_data"]

    updated_at: datetime


class IntegrationProfileSharedEntryDocumentFingerprintEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["document_fingerprint"]

    updated_at: datetime


class IntegrationProfileSharedEntryWordListEntry(BaseModel):
    id: str

    created_at: datetime

    enabled: bool

    name: str

    type: Literal["word_list"]

    updated_at: datetime

    word_list: object

    profile_id: Optional[str] = None


IntegrationProfileSharedEntry: TypeAlias = Union[
    IntegrationProfileSharedEntryCustomEntry,
    IntegrationProfileSharedEntryCustomPromptTopicEntry,
    IntegrationProfileSharedEntryPredefinedEntry,
    IntegrationProfileSharedEntryIntegrationEntry,
    IntegrationProfileSharedEntryExactDataEntry,
    IntegrationProfileSharedEntryDocumentFingerprintEntry,
    IntegrationProfileSharedEntryWordListEntry,
]


class IntegrationProfile(BaseModel):
    id: str

    created_at: datetime

    entries: List[IntegrationProfileEntry]

    name: str

    shared_entries: List[IntegrationProfileSharedEntry]

    type: Literal["integration"]

    updated_at: datetime

    description: Optional[str] = None
    """The description of the profile."""


Profile: TypeAlias = Union[CustomProfile, PredefinedProfile, IntegrationProfile]

from .profiles.pattern import Pattern
from .context_awareness import ContextAwareness

if PYDANTIC_V2:
    CustomProfile.model_rebuild()
    CustomProfileEntryCustomEntry.model_rebuild()
    CustomProfileEntryPredefinedEntry.model_rebuild()
    CustomProfileEntryPredefinedEntryConfidence.model_rebuild()
    CustomProfileEntryIntegrationEntry.model_rebuild()
    CustomProfileEntryExactDataEntry.model_rebuild()
    CustomProfileEntryWordListEntry.model_rebuild()
    CustomProfileSharedEntryCustomEntry.model_rebuild()
    PredefinedProfile.model_rebuild()
    PredefinedProfileEntryCustomEntry.model_rebuild()
    PredefinedProfileEntryPredefinedEntry.model_rebuild()
    PredefinedProfileEntryPredefinedEntryConfidence.model_rebuild()
    PredefinedProfileEntryIntegrationEntry.model_rebuild()
    PredefinedProfileEntryExactDataEntry.model_rebuild()
    PredefinedProfileEntryWordListEntry.model_rebuild()
    IntegrationProfile.model_rebuild()
    IntegrationProfileEntryCustomEntry.model_rebuild()
    IntegrationProfileEntryPredefinedEntry.model_rebuild()
    IntegrationProfileEntryPredefinedEntryConfidence.model_rebuild()
    IntegrationProfileEntryIntegrationEntry.model_rebuild()
    IntegrationProfileEntryExactDataEntry.model_rebuild()
    IntegrationProfileEntryWordListEntry.model_rebuild()
    IntegrationProfileSharedEntryCustomEntry.model_rebuild()
else:
    CustomProfile.update_forward_refs()  # type: ignore
    CustomProfileEntryCustomEntry.update_forward_refs()  # type: ignore
    CustomProfileEntryPredefinedEntry.update_forward_refs()  # type: ignore
    CustomProfileEntryPredefinedEntryConfidence.update_forward_refs()  # type: ignore
    CustomProfileEntryIntegrationEntry.update_forward_refs()  # type: ignore
    CustomProfileEntryExactDataEntry.update_forward_refs()  # type: ignore
    CustomProfileEntryWordListEntry.update_forward_refs()  # type: ignore
    CustomProfileSharedEntryCustomEntry.update_forward_refs()  # type: ignore
    PredefinedProfile.update_forward_refs()  # type: ignore
    PredefinedProfileEntryCustomEntry.update_forward_refs()  # type: ignore
    PredefinedProfileEntryPredefinedEntry.update_forward_refs()  # type: ignore
    PredefinedProfileEntryPredefinedEntryConfidence.update_forward_refs()  # type: ignore
    PredefinedProfileEntryIntegrationEntry.update_forward_refs()  # type: ignore
    PredefinedProfileEntryExactDataEntry.update_forward_refs()  # type: ignore
    PredefinedProfileEntryWordListEntry.update_forward_refs()  # type: ignore
    IntegrationProfile.update_forward_refs()  # type: ignore
    IntegrationProfileEntryCustomEntry.update_forward_refs()  # type: ignore
    IntegrationProfileEntryPredefinedEntry.update_forward_refs()  # type: ignore
    IntegrationProfileEntryPredefinedEntryConfidence.update_forward_refs()  # type: ignore
    IntegrationProfileEntryIntegrationEntry.update_forward_refs()  # type: ignore
    IntegrationProfileEntryExactDataEntry.update_forward_refs()  # type: ignore
    IntegrationProfileEntryWordListEntry.update_forward_refs()  # type: ignore
    IntegrationProfileSharedEntryCustomEntry.update_forward_refs()  # type: ignore



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["History"]


class History(BaseModel):
    id: Optional[str] = None
    """UUID"""

    alert_body: Optional[str] = None
    """Message body included in the notification sent."""

    alert_type: Optional[str] = None
    """Type of notification that has been dispatched."""

    description: Optional[str] = None
    """Description of the notification policy (if present)."""

    mechanism: Optional[str] = None
    """The mechanism to which the notification has been dispatched."""

    mechanism_type: Optional[Literal["email", "pagerduty", "webhook"]] = None
    """The type of mechanism to which the notification has been dispatched.

    This can be email/pagerduty/webhook based on the mechanism configured.
    """

    name: Optional[str] = None
    """Name of the policy."""

    policy_id: Optional[str] = None
    """The unique identifier of a notification policy"""

    sent: Optional[datetime] = None
    """Timestamp of when the notification was dispatched in ISO 8601 format."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "RuleBulkEditParams",
    "Body",
    "BodyPosition",
    "BodyPositionAPIShieldIndex",
    "BodyPositionAPIShieldBefore",
    "BodyPositionAPIShieldAfter",
    "BodySelector",
    "BodySelectorExclude",
    "BodySelectorInclude",
]


class RuleBulkEditParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    body: Required[Iterable[Body]]


class BodyPositionAPIShieldIndex(TypedDict, total=False):
    index: Required[int]
    """Move rule to this position"""


class BodyPositionAPIShieldBefore(TypedDict, total=False):
    """Move rule to after rule with ID."""

    before: str
    """Move rule to before rule with this ID."""


class BodyPositionAPIShieldAfter(TypedDict, total=False):
    """Move rule to before rule with ID."""

    after: str
    """Move rule to after rule with this ID."""


BodyPosition: TypeAlias = Union[BodyPositionAPIShieldIndex, BodyPositionAPIShieldBefore, BodyPositionAPIShieldAfter]


class BodySelectorExclude(TypedDict, total=False):
    operation_ids: SequenceNotStr[str]
    """Excluded operation IDs."""


class BodySelectorInclude(TypedDict, total=False):
    host: SequenceNotStr[str]
    """Included hostnames."""


class BodySelector(TypedDict, total=False):
    """Select operations covered by this rule.

    For details on selectors, see the [Cloudflare Docs](https://developers.cloudflare.com/api-shield/security/jwt-validation/).
    """

    exclude: Optional[Iterable[BodySelectorExclude]]
    """Ignore operations that were otherwise included by `include`."""

    include: Optional[Iterable[BodySelectorInclude]]
    """Select all matching operations."""


class Body(TypedDict, total=False):
    id: Required[str]
    """Rule ID this patch applies to"""

    action: Literal["log", "block"]
    """
    Action to take on requests that match operations included in `selector` and fail
    `expression`.
    """

    description: str
    """A human-readable description that gives more details than `title`."""

    enabled: bool
    """Toggle rule on or off."""

    expression: str
    """Rule expression.

    Requests that fail to match this expression will be subject to `action`.

    For details on expressions, see the
    [Cloudflare Docs](https://developers.cloudflare.com/api-shield/security/jwt-validation/).
    """

    position: BodyPosition
    """Update rule order among zone rules."""

    selector: BodySelector
    """Select operations covered by this rule.

    For details on selectors, see the
    [Cloudflare Docs](https://developers.cloudflare.com/api-shield/security/jwt-validation/).
    """

    title: str
    """A human-readable name for the rule."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .pages import (
    PagesResource,
    AsyncPagesResource,
    PagesResourceWithRawResponse,
    AsyncPagesResourceWithRawResponse,
    PagesResourceWithStreamingResponse,
    AsyncPagesResourceWithStreamingResponse,
)
from .tests import (
    TestsResource,
    AsyncTestsResource,
    TestsResourceWithRawResponse,
    AsyncTestsResourceWithRawResponse,
    TestsResourceWithStreamingResponse,
    AsyncTestsResourceWithStreamingResponse,
)

__all__ = [
    "TestsResource",
    "AsyncTestsResource",
    "TestsResourceWithRawResponse",
    "AsyncTestsResourceWithRawResponse",
    "TestsResourceWithStreamingResponse",
    "AsyncTestsResourceWithStreamingResponse",
    "PagesResource",
    "AsyncPagesResource",
    "PagesResourceWithRawResponse",
    "AsyncPagesResourceWithRawResponse",
    "PagesResourceWithStreamingResponse",
    "AsyncPagesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .mcp import (
    McpResource,
    AsyncMcpResource,
    McpResourceWithRawResponse,
    AsyncMcpResourceWithRawResponse,
    McpResourceWithStreamingResponse,
    AsyncMcpResourceWithStreamingResponse,
)
from .ai_controls import (
    AIControlsResource,
    AsyncAIControlsResource,
    AIControlsResourceWithRawResponse,
    AsyncAIControlsResourceWithRawResponse,
    AIControlsResourceWithStreamingResponse,
    AsyncAIControlsResourceWithStreamingResponse,
)

__all__ = [
    "McpResource",
    "AsyncMcpResource",
    "McpResourceWithRawResponse",
    "AsyncMcpResourceWithRawResponse",
    "McpResourceWithStreamingResponse",
    "AsyncMcpResourceWithStreamingResponse",
    "AIControlsResource",
    "AsyncAIControlsResource",
    "AIControlsResourceWithRawResponse",
    "AsyncAIControlsResourceWithRawResponse",
    "AIControlsResourceWithStreamingResponse",
    "AsyncAIControlsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AlwaysUseHTTPS"]


class AlwaysUseHTTPS(BaseModel):
    id: Optional[Literal["always_use_https"]] = None
    """
    If enabled, any ` http://`` URL is converted to  `https://` through a 301
    redirect.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ...._types import SequenceNotStr

__all__ = ["FallthroughCreateParams"]


class FallthroughCreateParams(TypedDict, total=False):
    zone_id: Required[str]
    """Identifier."""

    hosts: Required[SequenceNotStr[str]]
    """List of hosts to be targeted in the expression"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .rum_rule import RUMRule
from ..._models import BaseModel

__all__ = ["RuleListResponse", "Ruleset"]


class Ruleset(BaseModel):
    id: Optional[str] = None
    """The Web Analytics ruleset identifier."""

    enabled: Optional[bool] = None
    """Whether the ruleset is enabled."""

    zone_name: Optional[str] = None

    zone_tag: Optional[str] = None
    """The zone identifier."""


class RuleListResponse(BaseModel):
    rules: Optional[List[RUMRule]] = None
    """A list of rules."""

    ruleset: Optional[Ruleset] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["KeyUpdateParams"]


class KeyUpdateParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    key_rotation_interval_days: Required[float]
    """The number of days between key rotations."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import TypedDict

__all__ = ["ExtendedEmailMatchingParam"]


class ExtendedEmailMatchingParam(TypedDict, total=False):
    """Configures user email settings for firewall policies.

    When you enable this, the system standardizes email addresses in the identity portion of the rule to match extended email variants in firewall policies. When you disable this setting, the system matches email addresses exactly as you provide them. Enable this setting if your email uses `.` or `+` modifiers.
    """

    enabled: Optional[bool]
    """Specify whether to match all variants of user emails (with + or .

    modifiers) used as criteria in Firewall policies.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ....._utils import PropertyInfo

__all__ = ["ASPAChangesParams"]


class ASPAChangesParams(TypedDict, total=False):
    asn: int
    """Filter changes involving this ASN (as customer or provider)."""

    date_end: Annotated[Union[str, datetime], PropertyInfo(alias="dateEnd", format="iso8601")]
    """End of the date range (inclusive)."""

    date_start: Annotated[Union[str, datetime], PropertyInfo(alias="dateStart", format="iso8601")]
    """Start of the date range (inclusive)."""

    format: Literal["JSON", "CSV"]
    """Format in which results will be returned."""

    include_asn_info: Annotated[bool, PropertyInfo(alias="includeAsnInfo")]
    """Include ASN metadata (name, country) in response."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["UARuleListParams"]


class UARuleListParams(TypedDict, total=False):
    zone_id: Required[str]
    """Defines an identifier."""

    description: str
    """A string to search for in the description of existing rules."""

    page: float
    """Page number of paginated results."""

    paused: bool
    """When true, indicates that the rule is currently paused."""

    per_page: float
    """The maximum number of results per page.

    You can only set the value to `1` or to a multiple of 5 such as `5`, `10`, `15`,
    or `20`.
    """

    user_agent: str
    """A string to search for in the user agent values of existing rules."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .latest import (
    LatestResource,
    AsyncLatestResource,
    LatestResourceWithRawResponse,
    AsyncLatestResourceWithRawResponse,
    LatestResourceWithStreamingResponse,
    AsyncLatestResourceWithStreamingResponse,
)
from .snapshots import (
    SnapshotsResource,
    AsyncSnapshotsResource,
    SnapshotsResourceWithRawResponse,
    AsyncSnapshotsResourceWithRawResponse,
    SnapshotsResourceWithStreamingResponse,
    AsyncSnapshotsResourceWithStreamingResponse,
)

__all__ = [
    "LatestResource",
    "AsyncLatestResource",
    "LatestResourceWithRawResponse",
    "AsyncLatestResourceWithRawResponse",
    "LatestResourceWithStreamingResponse",
    "AsyncLatestResourceWithStreamingResponse",
    "SnapshotsResource",
    "AsyncSnapshotsResource",
    "SnapshotsResourceWithRawResponse",
    "AsyncSnapshotsResourceWithRawResponse",
    "SnapshotsResourceWithStreamingResponse",
    "AsyncSnapshotsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .jobs.jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from .connectivity_precheck import (
    ConnectivityPrecheckResource,
    AsyncConnectivityPrecheckResource,
    ConnectivityPrecheckResourceWithRawResponse,
    AsyncConnectivityPrecheckResourceWithRawResponse,
    ConnectivityPrecheckResourceWithStreamingResponse,
    AsyncConnectivityPrecheckResourceWithStreamingResponse,
)

__all__ = ["SuperSlurperResource", "AsyncSuperSlurperResource"]


class SuperSlurperResource(SyncAPIResource):
    @cached_property
    def jobs(self) -> JobsResource:
        return JobsResource(self._client)

    @cached_property
    def connectivity_precheck(self) -> ConnectivityPrecheckResource:
        return ConnectivityPrecheckResource(self._client)

    @cached_property
    def with_raw_response(self) -> SuperSlurperResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SuperSlurperResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SuperSlurperResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SuperSlurperResourceWithStreamingResponse(self)


class AsyncSuperSlurperResource(AsyncAPIResource):
    @cached_property
    def jobs(self) -> AsyncJobsResource:
        return AsyncJobsResource(self._client)

    @cached_property
    def connectivity_precheck(self) -> AsyncConnectivityPrecheckResource:
        return AsyncConnectivityPrecheckResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncSuperSlurperResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSuperSlurperResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSuperSlurperResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSuperSlurperResourceWithStreamingResponse(self)


class SuperSlurperResourceWithRawResponse:
    def __init__(self, super_slurper: SuperSlurperResource) -> None:
        self._super_slurper = super_slurper

    @cached_property
    def jobs(self) -> JobsResourceWithRawResponse:
        return JobsResourceWithRawResponse(self._super_slurper.jobs)

    @cached_property
    def connectivity_precheck(self) -> ConnectivityPrecheckResourceWithRawResponse:
        return ConnectivityPrecheckResourceWithRawResponse(self._super_slurper.connectivity_precheck)


class AsyncSuperSlurperResourceWithRawResponse:
    def __init__(self, super_slurper: AsyncSuperSlurperResource) -> None:
        self._super_slurper = super_slurper

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithRawResponse:
        return AsyncJobsResourceWithRawResponse(self._super_slurper.jobs)

    @cached_property
    def connectivity_precheck(self) -> AsyncConnectivityPrecheckResourceWithRawResponse:
        return AsyncConnectivityPrecheckResourceWithRawResponse(self._super_slurper.connectivity_precheck)


class SuperSlurperResourceWithStreamingResponse:
    def __init__(self, super_slurper: SuperSlurperResource) -> None:
        self._super_slurper = super_slurper

    @cached_property
    def jobs(self) -> JobsResourceWithStreamingResponse:
        return JobsResourceWithStreamingResponse(self._super_slurper.jobs)

    @cached_property
    def connectivity_precheck(self) -> ConnectivityPrecheckResourceWithStreamingResponse:
        return ConnectivityPrecheckResourceWithStreamingResponse(self._super_slurper.connectivity_precheck)


class AsyncSuperSlurperResourceWithStreamingResponse:
    def __init__(self, super_slurper: AsyncSuperSlurperResource) -> None:
        self._super_slurper = super_slurper

    @cached_property
    def jobs(self) -> AsyncJobsResourceWithStreamingResponse:
        return AsyncJobsResourceWithStreamingResponse(self._super_slurper.jobs)

    @cached_property
    def connectivity_precheck(self) -> AsyncConnectivityPrecheckResourceWithStreamingResponse:
        return AsyncConnectivityPrecheckResourceWithStreamingResponse(self._super_slurper.connectivity_precheck)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .limits import (
    LimitsResource,
    AsyncLimitsResource,
    LimitsResourceWithRawResponse,
    AsyncLimitsResourceWithRawResponse,
    LimitsResourceWithStreamingResponse,
    AsyncLimitsResourceWithStreamingResponse,
)
from .patterns import (
    PatternsResource,
    AsyncPatternsResource,
    PatternsResourceWithRawResponse,
    AsyncPatternsResourceWithRawResponse,
    PatternsResourceWithStreamingResponse,
    AsyncPatternsResourceWithStreamingResponse,
)
from .settings import (
    SettingsResource,
    AsyncSettingsResource,
    SettingsResourceWithRawResponse,
    AsyncSettingsResourceWithRawResponse,
    SettingsResourceWithStreamingResponse,
    AsyncSettingsResourceWithStreamingResponse,
)
from ...._compat import cached_property
from .email.email import (
    EmailResource,
    AsyncEmailResource,
    EmailResourceWithRawResponse,
    AsyncEmailResourceWithRawResponse,
    EmailResourceWithStreamingResponse,
    AsyncEmailResourceWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from .payload_logs import (
    PayloadLogsResource,
    AsyncPayloadLogsResource,
    PayloadLogsResourceWithRawResponse,
    AsyncPayloadLogsResourceWithRawResponse,
    PayloadLogsResourceWithStreamingResponse,
    AsyncPayloadLogsResourceWithStreamingResponse,
)
from .entries.entries import (
    EntriesResource,
    AsyncEntriesResource,
    EntriesResourceWithRawResponse,
    AsyncEntriesResourceWithRawResponse,
    EntriesResourceWithStreamingResponse,
    AsyncEntriesResourceWithStreamingResponse,
)
from .datasets.datasets import (
    DatasetsResource,
    AsyncDatasetsResource,
    DatasetsResourceWithRawResponse,
    AsyncDatasetsResourceWithRawResponse,
    DatasetsResourceWithStreamingResponse,
    AsyncDatasetsResourceWithStreamingResponse,
)
from .profiles.profiles import (
    ProfilesResource,
    AsyncProfilesResource,
    ProfilesResourceWithRawResponse,
    AsyncProfilesResourceWithRawResponse,
    ProfilesResourceWithStreamingResponse,
    AsyncProfilesResourceWithStreamingResponse,
)

__all__ = ["DLPResource", "AsyncDLPResource"]


class DLPResource(SyncAPIResource):
    @cached_property
    def datasets(self) -> DatasetsResource:
        return DatasetsResource(self._client)

    @cached_property
    def patterns(self) -> PatternsResource:
        return PatternsResource(self._client)

    @cached_property
    def payload_logs(self) -> PayloadLogsResource:
        return PayloadLogsResource(self._client)

    @cached_property
    def settings(self) -> SettingsResource:
        return SettingsResource(self._client)

    @cached_property
    def email(self) -> EmailResource:
        return EmailResource(self._client)

    @cached_property
    def profiles(self) -> ProfilesResource:
        return ProfilesResource(self._client)

    @cached_property
    def limits(self) -> LimitsResource:
        return LimitsResource(self._client)

    @cached_property
    def entries(self) -> EntriesResource:
        return EntriesResource(self._client)

    @cached_property
    def with_raw_response(self) -> DLPResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DLPResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DLPResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DLPResourceWithStreamingResponse(self)


class AsyncDLPResource(AsyncAPIResource):
    @cached_property
    def datasets(self) -> AsyncDatasetsResource:
        return AsyncDatasetsResource(self._client)

    @cached_property
    def patterns(self) -> AsyncPatternsResource:
        return AsyncPatternsResource(self._client)

    @cached_property
    def payload_logs(self) -> AsyncPayloadLogsResource:
        return AsyncPayloadLogsResource(self._client)

    @cached_property
    def settings(self) -> AsyncSettingsResource:
        return AsyncSettingsResource(self._client)

    @cached_property
    def email(self) -> AsyncEmailResource:
        return AsyncEmailResource(self._client)

    @cached_property
    def profiles(self) -> AsyncProfilesResource:
        return AsyncProfilesResource(self._client)

    @cached_property
    def limits(self) -> AsyncLimitsResource:
        return AsyncLimitsResource(self._client)

    @cached_property
    def entries(self) -> AsyncEntriesResource:
        return AsyncEntriesResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncDLPResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDLPResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDLPResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDLPResourceWithStreamingResponse(self)


class DLPResourceWithRawResponse:
    def __init__(self, dlp: DLPResource) -> None:
        self._dlp = dlp

    @cached_property
    def datasets(self) -> DatasetsResourceWithRawResponse:
        return DatasetsResourceWithRawResponse(self._dlp.datasets)

    @cached_property
    def patterns(self) -> PatternsResourceWithRawResponse:
        return PatternsResourceWithRawResponse(self._dlp.patterns)

    @cached_property
    def payload_logs(self) -> PayloadLogsResourceWithRawResponse:
        return PayloadLogsResourceWithRawResponse(self._dlp.payload_logs)

    @cached_property
    def settings(self) -> SettingsResourceWithRawResponse:
        return SettingsResourceWithRawResponse(self._dlp.settings)

    @cached_property
    def email(self) -> EmailResourceWithRawResponse:
        return EmailResourceWithRawResponse(self._dlp.email)

    @cached_property
    def profiles(self) -> ProfilesResourceWithRawResponse:
        return ProfilesResourceWithRawResponse(self._dlp.profiles)

    @cached_property
    def limits(self) -> LimitsResourceWithRawResponse:
        return LimitsResourceWithRawResponse(self._dlp.limits)

    @cached_property
    def entries(self) -> EntriesResourceWithRawResponse:
        return EntriesResourceWithRawResponse(self._dlp.entries)


class AsyncDLPResourceWithRawResponse:
    def __init__(self, dlp: AsyncDLPResource) -> None:
        self._dlp = dlp

    @cached_property
    def datasets(self) -> AsyncDatasetsResourceWithRawResponse:
        return AsyncDatasetsResourceWithRawResponse(self._dlp.datasets)

    @cached_property
    def patterns(self) -> AsyncPatternsResourceWithRawResponse:
        return AsyncPatternsResourceWithRawResponse(self._dlp.patterns)

    @cached_property
    def payload_logs(self) -> AsyncPayloadLogsResourceWithRawResponse:
        return AsyncPayloadLogsResourceWithRawResponse(self._dlp.payload_logs)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithRawResponse:
        return AsyncSettingsResourceWithRawResponse(self._dlp.settings)

    @cached_property
    def email(self) -> AsyncEmailResourceWithRawResponse:
        return AsyncEmailResourceWithRawResponse(self._dlp.email)

    @cached_property
    def profiles(self) -> AsyncProfilesResourceWithRawResponse:
        return AsyncProfilesResourceWithRawResponse(self._dlp.profiles)

    @cached_property
    def limits(self) -> AsyncLimitsResourceWithRawResponse:
        return AsyncLimitsResourceWithRawResponse(self._dlp.limits)

    @cached_property
    def entries(self) -> AsyncEntriesResourceWithRawResponse:
        return AsyncEntriesResourceWithRawResponse(self._dlp.entries)


class DLPResourceWithStreamingResponse:
    def __init__(self, dlp: DLPResource) -> None:
        self._dlp = dlp

    @cached_property
    def datasets(self) -> DatasetsResourceWithStreamingResponse:
        return DatasetsResourceWithStreamingResponse(self._dlp.datasets)

    @cached_property
    def patterns(self) -> PatternsResourceWithStreamingResponse:
        return PatternsResourceWithStreamingResponse(self._dlp.patterns)

    @cached_property
    def payload_logs(self) -> PayloadLogsResourceWithStreamingResponse:
        return PayloadLogsResourceWithStreamingResponse(self._dlp.payload_logs)

    @cached_property
    def settings(self) -> SettingsResourceWithStreamingResponse:
        return SettingsResourceWithStreamingResponse(self._dlp.settings)

    @cached_property
    def email(self) -> EmailResourceWithStreamingResponse:
        return EmailResourceWithStreamingResponse(self._dlp.email)

    @cached_property
    def profiles(self) -> ProfilesResourceWithStreamingResponse:
        return ProfilesResourceWithStreamingResponse(self._dlp.profiles)

    @cached_property
    def limits(self) -> LimitsResourceWithStreamingResponse:
        return LimitsResourceWithStreamingResponse(self._dlp.limits)

    @cached_property
    def entries(self) -> EntriesResourceWithStreamingResponse:
        return EntriesResourceWithStreamingResponse(self._dlp.entries)


class AsyncDLPResourceWithStreamingResponse:
    def __init__(self, dlp: AsyncDLPResource) -> None:
        self._dlp = dlp

    @cached_property
    def datasets(self) -> AsyncDatasetsResourceWithStreamingResponse:
        return AsyncDatasetsResourceWithStreamingResponse(self._dlp.datasets)

    @cached_property
    def patterns(self) -> AsyncPatternsResourceWithStreamingResponse:
        return AsyncPatternsResourceWithStreamingResponse(self._dlp.patterns)

    @cached_property
    def payload_logs(self) -> AsyncPayloadLogsResourceWithStreamingResponse:
        return AsyncPayloadLogsResourceWithStreamingResponse(self._dlp.payload_logs)

    @cached_property
    def settings(self) -> AsyncSettingsResourceWithStreamingResponse:
        return AsyncSettingsResourceWithStreamingResponse(self._dlp.settings)

    @cached_property
    def email(self) -> AsyncEmailResourceWithStreamingResponse:
        return AsyncEmailResourceWithStreamingResponse(self._dlp.email)

    @cached_property
    def profiles(self) -> AsyncProfilesResourceWithStreamingResponse:
        return AsyncProfilesResourceWithStreamingResponse(self._dlp.profiles)

    @cached_property
    def limits(self) -> AsyncLimitsResourceWithStreamingResponse:
        return AsyncLimitsResourceWithStreamingResponse(self._dlp.limits)

    @cached_property
    def entries(self) -> AsyncEntriesResourceWithStreamingResponse:
        return AsyncEntriesResourceWithStreamingResponse(self._dlp.entries)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr

__all__ = [
    "SubscriptionCreateParams",
    "Destination",
    "Source",
    "SourceMqEventSourceImages",
    "SourceMqEventSourceKV",
    "SourceMqEventSourceR2",
    "SourceMqEventSourceSuperSlurper",
    "SourceMqEventSourceVectorize",
    "SourceMqEventSourceWorkersAIModel",
    "SourceMqEventSourceWorkersBuildsWorker",
    "SourceMqEventSourceWorkflowsWorkflow",
]


class SubscriptionCreateParams(TypedDict, total=False):
    account_id: Required[str]
    """A Resource identifier."""

    destination: Destination
    """Destination configuration for the subscription"""

    enabled: bool
    """Whether the subscription is active"""

    events: SequenceNotStr[str]
    """List of event types this subscription handles"""

    name: str
    """Name of the subscription"""

    source: Source
    """Source configuration for the subscription"""


class Destination(TypedDict, total=False):
    """Destination configuration for the subscription"""

    queue_id: Required[str]
    """ID of the target queue"""

    type: Required[Literal["queues.queue"]]
    """Type of destination"""


class SourceMqEventSourceImages(TypedDict, total=False):
    type: Literal["images"]
    """Type of source"""


class SourceMqEventSourceKV(TypedDict, total=False):
    type: Literal["kv"]
    """Type of source"""


class SourceMqEventSourceR2(TypedDict, total=False):
    type: Literal["r2"]
    """Type of source"""


class SourceMqEventSourceSuperSlurper(TypedDict, total=False):
    type: Literal["superSlurper"]
    """Type of source"""


class SourceMqEventSourceVectorize(TypedDict, total=False):
    type: Literal["vectorize"]
    """Type of source"""


class SourceMqEventSourceWorkersAIModel(TypedDict, total=False):
    model_name: str
    """Name of the Workers AI model"""

    type: Literal["workersAi.model"]
    """Type of source"""


class SourceMqEventSourceWorkersBuildsWorker(TypedDict, total=False):
    type: Literal["workersBuilds.worker"]
    """Type of source"""

    worker_name: str
    """Name of the worker"""


class SourceMqEventSourceWorkflowsWorkflow(TypedDict, total=False):
    type: Literal["workflows.workflow"]
    """Type of source"""

    workflow_name: str
    """Name of the workflow"""


Source: TypeAlias = Union[
    SourceMqEventSourceImages,
    SourceMqEventSourceKV,
    SourceMqEventSourceR2,
    SourceMqEventSourceSuperSlurper,
    SourceMqEventSourceVectorize,
    SourceMqEventSourceWorkersAIModel,
    SourceMqEventSourceWorkersBuildsWorker,
    SourceMqEventSourceWorkflowsWorkflow,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from ..._models import BaseModel

__all__ = ["MessagePullResponse", "Message", "Metadata", "MetadataMetrics"]


class Message(BaseModel):
    id: Optional[str] = None

    attempts: Optional[float] = None

    body: Optional[str] = None

    lease_id: Optional[str] = None
    """An ID that represents an "in-flight" message that has been pulled from a Queue.

    You must hold on to this ID and use it to acknowledge this message.
    """

    metadata: Optional[object] = None

    timestamp_ms: Optional[float] = None


class MetadataMetrics(BaseModel):
    """Best-effort metrics for the queue.

    Values may be approximate due to the distributed nature of queues.
    """

    backlog_bytes: float
    """The size in bytes of unacknowledged messages in the queue."""

    backlog_count: float
    """The number of unacknowledged messages in the queue."""

    oldest_message_timestamp_ms: float
    """Unix timestamp in milliseconds of the oldest unacknowledged message in the
    queue.

    Returns 0 if unknown.
    """


class Metadata(BaseModel):
    metrics: Optional[MetadataMetrics] = None
    """Best-effort metrics for the queue.

    Values may be approximate due to the distributed nature of queues.
    """


class MessagePullResponse(BaseModel):
    message_backlog_count: Optional[float] = None
    """The number of unacknowledged messages in the queue."""

    messages: Optional[List[Message]] = None

    metadata: Optional[Metadata] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .top import (
    TopResource,
    AsyncTopResource,
    TopResourceWithRawResponse,
    AsyncTopResourceWithRawResponse,
    TopResourceWithStreamingResponse,
    AsyncTopResourceWithStreamingResponse,
)
from .robots_txt import (
    RobotsTXTResource,
    AsyncRobotsTXTResource,
    RobotsTXTResourceWithRawResponse,
    AsyncRobotsTXTResourceWithRawResponse,
    RobotsTXTResourceWithStreamingResponse,
    AsyncRobotsTXTResourceWithStreamingResponse,
)

__all__ = [
    "TopResource",
    "AsyncTopResource",
    "TopResourceWithRawResponse",
    "AsyncTopResourceWithRawResponse",
    "TopResourceWithStreamingResponse",
    "AsyncTopResourceWithStreamingResponse",
    "RobotsTXTResource",
    "AsyncRobotsTXTResource",
    "RobotsTXTResourceWithRawResponse",
    "AsyncRobotsTXTResourceWithRawResponse",
    "RobotsTXTResourceWithStreamingResponse",
    "AsyncRobotsTXTResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["DomainJoinedInput"]


class DomainJoinedInput(BaseModel):
    operating_system: Literal["windows"]
    """Operating System."""

    domain: Optional[str] = None
    """Domain."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["Matcher"]


class Matcher(BaseModel):
    """Matching pattern to forward your actions."""

    type: Literal["all", "literal"]
    """Type of matcher."""

    field: Optional[Literal["to"]] = None
    """Field for type matcher."""

    value: Optional[str] = None
    """Value for matcher."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import path_template, maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...._wrappers import ResultWrapper
from ...._base_client import make_request_options
from ....types.email_security.investigate import reclassify_create_params

__all__ = ["ReclassifyResource", "AsyncReclassifyResource"]


class ReclassifyResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ReclassifyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return ReclassifyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReclassifyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return ReclassifyResourceWithStreamingResponse(self)

    def create(
        self,
        investigate_id: str,
        *,
        account_id: str,
        expected_disposition: Literal["NONE", "BULK", "MALICIOUS", "SPAM", "SPOOF", "SUSPICIOUS"],
        eml_content: str | Omit = omit,
        escalated_submission_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Submits a request to reclassify an email's disposition.

        Use for reporting false
        positives or false negatives. Optionally provide the raw EML content for
        reanalysis. The reclassification is processed asynchronously.

        Args:
          account_id: Identifier.

          investigate_id: Unique identifier for a message retrieved from investigation

          eml_content: Base64 encoded content of the EML file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not investigate_id:
            raise ValueError(f"Expected a non-empty value for `investigate_id` but received {investigate_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/email-security/investigate/{investigate_id}/reclassify",
                account_id=account_id,
                investigate_id=investigate_id,
            ),
            body=maybe_transform(
                {
                    "expected_disposition": expected_disposition,
                    "eml_content": eml_content,
                    "escalated_submission_id": escalated_submission_id,
                },
                reclassify_create_params.ReclassifyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class AsyncReclassifyResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncReclassifyResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncReclassifyResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReclassifyResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncReclassifyResourceWithStreamingResponse(self)

    async def create(
        self,
        investigate_id: str,
        *,
        account_id: str,
        expected_disposition: Literal["NONE", "BULK", "MALICIOUS", "SPAM", "SPOOF", "SUSPICIOUS"],
        eml_content: str | Omit = omit,
        escalated_submission_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """Submits a request to reclassify an email's disposition.

        Use for reporting false
        positives or false negatives. Optionally provide the raw EML content for
        reanalysis. The reclassification is processed asynchronously.

        Args:
          account_id: Identifier.

          investigate_id: Unique identifier for a message retrieved from investigation

          eml_content: Base64 encoded content of the EML file.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not investigate_id:
            raise ValueError(f"Expected a non-empty value for `investigate_id` but received {investigate_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/email-security/investigate/{investigate_id}/reclassify",
                account_id=account_id,
                investigate_id=investigate_id,
            ),
            body=await async_maybe_transform(
                {
                    "expected_disposition": expected_disposition,
                    "eml_content": eml_content,
                    "escalated_submission_id": escalated_submission_id,
                },
                reclassify_create_params.ReclassifyCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[object]._unwrapper,
            ),
            cast_to=cast(Type[object], ResultWrapper[object]),
        )


class ReclassifyResourceWithRawResponse:
    def __init__(self, reclassify: ReclassifyResource) -> None:
        self._reclassify = reclassify

        self.create = to_raw_response_wrapper(
            reclassify.create,
        )


class AsyncReclassifyResourceWithRawResponse:
    def __init__(self, reclassify: AsyncReclassifyResource) -> None:
        self._reclassify = reclassify

        self.create = async_to_raw_response_wrapper(
            reclassify.create,
        )


class ReclassifyResourceWithStreamingResponse:
    def __init__(self, reclassify: ReclassifyResource) -> None:
        self._reclassify = reclassify

        self.create = to_streamed_response_wrapper(
            reclassify.create,
        )


class AsyncReclassifyResourceWithStreamingResponse:
    def __init__(self, reclassify: AsyncReclassifyResource) -> None:
        self._reclassify = reclassify

        self.create = async_to_streamed_response_wrapper(
            reclassify.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from ...split_tunnel_exclude_param import SplitTunnelExcludeParam

__all__ = ["ExcludeUpdateParams"]


class ExcludeUpdateParams(TypedDict, total=False):
    account_id: Required[str]

    body: Required[Iterable[SplitTunnelExcludeParam]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ..._base_client import make_request_options
from ...types.logpush import ownership_create_params, ownership_validate_params
from ...types.logpush.ownership_validation import OwnershipValidation
from ...types.logpush.ownership_create_response import OwnershipCreateResponse

__all__ = ["OwnershipResource", "AsyncOwnershipResource"]


class OwnershipResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OwnershipResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OwnershipResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OwnershipResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OwnershipResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        destination_conf: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OwnershipCreateResponse]:
        """
        Gets a new ownership challenge sent to your destination.

        Args:
          destination_conf: Uniquely identifies a resource (such as an s3 bucket) where data. will be
              pushed. Additional configuration parameters supported by the destination may be
              included.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/ownership",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform({"destination_conf": destination_conf}, ownership_create_params.OwnershipCreateParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OwnershipCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OwnershipCreateResponse]], ResultWrapper[OwnershipCreateResponse]),
        )

    def validate(
        self,
        *,
        destination_conf: str,
        ownership_challenge: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OwnershipValidation]:
        """
        Validates ownership challenge of the destination.

        Args:
          destination_conf: Uniquely identifies a resource (such as an s3 bucket) where data. will be
              pushed. Additional configuration parameters supported by the destination may be
              included.

          ownership_challenge: Ownership challenge token to prove destination ownership.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/ownership/validate",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=maybe_transform(
                {
                    "destination_conf": destination_conf,
                    "ownership_challenge": ownership_challenge,
                },
                ownership_validate_params.OwnershipValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OwnershipValidation]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OwnershipValidation]], ResultWrapper[OwnershipValidation]),
        )


class AsyncOwnershipResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOwnershipResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOwnershipResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOwnershipResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOwnershipResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        destination_conf: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OwnershipCreateResponse]:
        """
        Gets a new ownership challenge sent to your destination.

        Args:
          destination_conf: Uniquely identifies a resource (such as an s3 bucket) where data. will be
              pushed. Additional configuration parameters supported by the destination may be
              included.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/ownership",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {"destination_conf": destination_conf}, ownership_create_params.OwnershipCreateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OwnershipCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OwnershipCreateResponse]], ResultWrapper[OwnershipCreateResponse]),
        )

    async def validate(
        self,
        *,
        destination_conf: str,
        ownership_challenge: str,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OwnershipValidation]:
        """
        Validates ownership challenge of the destination.

        Args:
          destination_conf: Uniquely identifies a resource (such as an s3 bucket) where data. will be
              pushed. Additional configuration parameters supported by the destination may be
              included.

          ownership_challenge: Ownership challenge token to prove destination ownership.

          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return await self._post(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/logpush/ownership/validate",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            body=await async_maybe_transform(
                {
                    "destination_conf": destination_conf,
                    "ownership_challenge": ownership_challenge,
                },
                ownership_validate_params.OwnershipValidateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OwnershipValidation]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OwnershipValidation]], ResultWrapper[OwnershipValidation]),
        )


class OwnershipResourceWithRawResponse:
    def __init__(self, ownership: OwnershipResource) -> None:
        self._ownership = ownership

        self.create = to_raw_response_wrapper(
            ownership.create,
        )
        self.validate = to_raw_response_wrapper(
            ownership.validate,
        )


class AsyncOwnershipResourceWithRawResponse:
    def __init__(self, ownership: AsyncOwnershipResource) -> None:
        self._ownership = ownership

        self.create = async_to_raw_response_wrapper(
            ownership.create,
        )
        self.validate = async_to_raw_response_wrapper(
            ownership.validate,
        )


class OwnershipResourceWithStreamingResponse:
    def __init__(self, ownership: OwnershipResource) -> None:
        self._ownership = ownership

        self.create = to_streamed_response_wrapper(
            ownership.create,
        )
        self.validate = to_streamed_response_wrapper(
            ownership.validate,
        )


class AsyncOwnershipResourceWithStreamingResponse:
    def __init__(self, ownership: AsyncOwnershipResource) -> None:
        self._ownership = ownership

        self.create = async_to_streamed_response_wrapper(
            ownership.create,
        )
        self.validate = async_to_streamed_response_wrapper(
            ownership.validate,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.intel import domain_history_get_params
from ..._base_client import make_request_options
from ...types.intel.domain_history_get_response import DomainHistoryGetResponse

__all__ = ["DomainHistoryResource", "AsyncDomainHistoryResource"]


class DomainHistoryResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DomainHistoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return DomainHistoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DomainHistoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return DomainHistoryResourceWithStreamingResponse(self)

    def get(
        self,
        *,
        account_id: str,
        domain: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DomainHistoryGetResponse]:
        """
        Gets historical security threat and content categories currently and previously
        assigned to a domain.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get(
            path_template("/accounts/{account_id}/intel/domain-history", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform({"domain": domain}, domain_history_get_params.DomainHistoryGetParams),
                post_parser=ResultWrapper[Optional[DomainHistoryGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DomainHistoryGetResponse]], ResultWrapper[DomainHistoryGetResponse]),
        )


class AsyncDomainHistoryResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDomainHistoryResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDomainHistoryResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDomainHistoryResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncDomainHistoryResourceWithStreamingResponse(self)

    async def get(
        self,
        *,
        account_id: str,
        domain: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[DomainHistoryGetResponse]:
        """
        Gets historical security threat and content categories currently and previously
        assigned to a domain.

        Args:
          account_id: Identifier.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._get(
            path_template("/accounts/{account_id}/intel/domain-history", account_id=account_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform({"domain": domain}, domain_history_get_params.DomainHistoryGetParams),
                post_parser=ResultWrapper[Optional[DomainHistoryGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[DomainHistoryGetResponse]], ResultWrapper[DomainHistoryGetResponse]),
        )


class DomainHistoryResourceWithRawResponse:
    def __init__(self, domain_history: DomainHistoryResource) -> None:
        self._domain_history = domain_history

        self.get = to_raw_response_wrapper(
            domain_history.get,
        )


class AsyncDomainHistoryResourceWithRawResponse:
    def __init__(self, domain_history: AsyncDomainHistoryResource) -> None:
        self._domain_history = domain_history

        self.get = async_to_raw_response_wrapper(
            domain_history.get,
        )


class DomainHistoryResourceWithStreamingResponse:
    def __init__(self, domain_history: DomainHistoryResource) -> None:
        self._domain_history = domain_history

        self.get = to_streamed_response_wrapper(
            domain_history.get,
        )


class AsyncDomainHistoryResourceWithStreamingResponse:
    def __init__(self, domain_history: AsyncDomainHistoryResource) -> None:
        self._domain_history = domain_history

        self.get = async_to_streamed_response_wrapper(
            domain_history.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .status_edit_params import StatusEditParams as StatusEditParams
from .event_create_params import EventCreateParams as EventCreateParams
from .status_edit_response import StatusEditResponse as StatusEditResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .setting_get_response import SettingGetResponse as SettingGetResponse
from .payload_create_params import PayloadCreateParams as PayloadCreateParams
from .payload_list_response import PayloadListResponse as PayloadListResponse
from .payload_create_response import PayloadCreateResponse as PayloadCreateResponse
from .payload_delete_response import PayloadDeleteResponse as PayloadDeleteResponse
from .content_scanning_get_response import ContentScanningGetResponse as ContentScanningGetResponse
from .content_scanning_create_params import ContentScanningCreateParams as ContentScanningCreateParams
from .content_scanning_update_params import ContentScanningUpdateParams as ContentScanningUpdateParams
from .content_scanning_create_response import ContentScanningCreateResponse as ContentScanningCreateResponse
from .content_scanning_update_response import ContentScanningUpdateResponse as ContentScanningUpdateResponse
